//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "SystemVector.h"
#include "Serializer.h"

namespace AMDiS {

  void SystemVector::serialize(std::ostream &out) 
  {
    int size = static_cast<int>(vectors.size());
    SerUtil::serialize(out, size);
    for (int i = 0; i < size; i++)
      vectors[i]->serialize(out);
  }


  void SystemVector::deserialize(std::istream &in) 
  {
    int size, oldSize = static_cast<int>(vectors.size());
    SerUtil::deserialize(in, size);
    vectors.resize(size);
    for (int i = oldSize; i < size; i++)
      vectors[i] = new DOFVector<double>(feSpace[i], "");
    for (int i = 0; i < size; i++)
      vectors[i]->deserialize(in);
  }


  vector<const FiniteElemSpace*> SystemVector::getComponentFeSpaces() const
  {
    vector<const FiniteElemSpace*> result(vectors.size());

    for (unsigned int i = 0; i < vectors.size(); i++)
      result[i] = vectors[i]->getFeSpace();

    return result;
  }
}
