// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscSolverFeti.h */

#ifndef AMDIS_PETSC_SOLVER_FETI_OPERATORS_H
#define AMDIS_PETSC_SOLVER_FETI_OPERATORS_H

#include <mpi.h>
#include <petsc.h>

namespace AMDiS {

  /// 
  int petscMultMatSchurPrimal(Mat mat, Vec x, Vec y);

  /// FETI-DP operator
  int petscMultMatFeti(Mat mat, Vec x, Vec y);

  /// FETI-DP operator used for Stokes like problems.
  int petscMultMatFetiInterface(Mat mat, Vec x, Vec y);

  /// y = PC * x
  PetscErrorCode petscApplyFetiDirichletPrecon(PC pc, Vec x, Vec y);

  /// y = PC * x
  PetscErrorCode petscApplyFetiLumpedPrecon(PC pc, Vec x, Vec y);

  PetscErrorCode petscApplyFetiInterfaceLumpedPrecon(PC pc, Vec x, Vec y);
}

#endif
