#include "PhaseFieldCrystal_RB.h"

using namespace std;
using namespace AMDiS;

PhaseFieldCrystal_RB::PhaseFieldCrystal_RB(const std::string &name_) :
  super(name_),
  useMobility(false),
  tempParameter(-0.6),
  r(-0.4),		// temperature deviation
  rho0(1.0),		// liquid density
  density(-0.3),	// mean density
  two(2.0),
  minus2(-2.0),
  minus1(-1.0)
{  
  Parameters::get(name + "->r",r);
  Parameters::get(name + "->rho0", rho0);
  Parameters::get(name + "->density", density);
  Parameters::get(name + "->use mobility", useMobility);
  tempParameter= -(1.0+r);
};


void PhaseFieldCrystal_RB::fillOperators()
{ FUNCNAME("PhaseFieldCrystal_RB::fillOperators()");
  
  // rho
  Operator *opMnu = new Operator(prob->getFeSpace(0), prob->getFeSpace(2));
  opMnu->addTerm(new Simple_ZOT);
  Operator *opMmu = new Operator(prob->getFeSpace(2), prob->getFeSpace(1));
  opMmu->addTerm(new Simple_ZOT);
  Operator *opMr = new Operator(prob->getFeSpace(2), prob->getFeSpace(0));
  opMr->addTerm(new Simple_ZOT(tempParameter));

  // -nabla*(grad(rho))
  Operator *opL = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL->addTerm(new Simple_SOT);
  Operator *opL2 = new Operator(prob->getFeSpace(2), prob->getFeSpace(0));
  opL2->addTerm(new Simple_SOT(2.0));
  Operator *opLnu = new Operator(prob->getFeSpace(2), prob->getFeSpace(2));
  opLnu->addTerm(new Simple_SOT);
  
  Operator *opLM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  if (useMobility) // non-constant mobility
    opLM->addTerm(new VecAtQP_SOT(
      prob->getSolution()->getDOFVector(0),
      new MobilityPfc(density, rho0)));
  else // constant mobility
    opLM->addTerm(new Simple_SOT);
  
  // -rho_old^3
  Operator *opMPowExpl = new Operator(prob->getFeSpace(2), prob->getFeSpace(0));
  opMPowExpl->addZeroOrderTerm(new VecAtQP_ZOT(
    prob->getStageSolution(0), new Pow3Functor(-1.0)));
  // 3*rho_old^2
  Operator *opMPowImpl = new Operator(prob->getFeSpace(2), prob->getFeSpace(0));
  opMPowImpl->addZeroOrderTerm(new VecAtQP_ZOT(
    prob->getUnVec(0), new Pow2Functor(3.0)));
  
  // 0 = nu-laplace(rho)
  // -------------------
  opL->setUhOld(prob->getStageSolution(0));
  prob->addVectorOperator(opL, 0); 		// -laplace(rho)
  prob->addMatrixOperator(opL, 0, 0, &minus1); 	// -laplace(rho)

  opMnu->setUhOld(prob->getStageSolution(2));
  prob->addVectorOperator(opMnu, 0); 			// nu
  prob->addMatrixOperator(opMnu, 0, 2, &minus1); 	// nu
  
  // dt(rho) = laplace(mu) - u*grad(rho)
  // -----------------------------------
  prob->addTimeOperator(1, 0);	// dt(rho)

  opLM->setUhOld(prob->getStageSolution(1));
  prob->addVectorOperator(opLM, 1, &minus1); 	// -laplace(mu)
  prob->addMatrixOperator(opLM, 1, 1); 		// -laplace(mu)
  
  // mu-2*nu-laplace(nu)-(1+r)*rho = (rho_old^3) + ExtPot - eps^2/(rho_old+0.9)
  // ----------------------------------------------------------------------
  opMr->setUhOld(prob->getStageSolution(0));
  prob->addVectorOperator(opMr, 2); 		// -(1+r)*rho
  prob->addMatrixOperator(opMr, 2, 0, &minus1); // -(1+r)*rho
  
  opL2->setUhOld(prob->getStageSolution(0));
  prob->addVectorOperator(opL2, 2); 		// -2*laplace(rho)
  prob->addMatrixOperator(opL2, 2, 0, &minus1); // -2*laplace(rho)

  opMmu->setUhOld(prob->getStageSolution(1));
  prob->addVectorOperator(opMmu, 2); 			// mu
  prob->addMatrixOperator(opMmu, 2, 1, &minus1); 	// mu

  opLnu->setUhOld(prob->getStageSolution(2));
  prob->addVectorOperator(opLnu, 2); 			// -laplace(nu)
  prob->addMatrixOperator(opLnu, 2, 2, &minus1); 	// -laplace(nu)

  // . . . nonlinear terms . . . . . . . . . . . . . . .
  prob->addMatrixOperator(opMPowImpl, 2, 0); 	// 3*rho*rho_old^2
  prob->addVectorOperator(opMPowExpl, 2); 	// -rho'^3
};


void PhaseFieldCrystal_RB::fillBoundaryConditions()
{ FUNCNAME("PhaseFieldCrystal_RB::fillBoundaryConditions()");
};
